(function () {
  "use strict";

  var namespace = require('ns3');
  var fs = require('fs');
  var EventEmitter = require('events');
  var util = require('util');
  var deepExtend = require('deep-extend');

  var Config = function (path1, path2) {
    if (!(path1 && path2)) {
      throw new Error('Config arguments "path1" and "path2" required.');
    }

    EventEmitter.call(this);

    this.isLoaded = false; // true - конфиг загружен
    this._paths = [path1, path2]; // Пути к файлам настроек
    this._data = null; // Данные настроек

  };
  util.inherits(Config, EventEmitter);

  Config.prototype.initialize = function (file1Data) {
    var path = file1Data ? this._paths[1] : this._paths[0];

    var data = null;
    // Загрузка файла
    try {
      data = fs.readFileSync(path, {encoding: 'utf-8'});
    } catch (err) {
      this.emit('error', err);
      return;
    }

    try {
      data = JSON.parse(data);
    } catch (e) {
      this.emit('error', e);
      return false;
    }

    if (file1Data) {
      // Успешно
      deepExtend(file1Data, data);

      this._data = file1Data;
      this.isLoaded = true;
      this.emit('load', file1Data);
      return true;
    } else {
      // Грузим второй файл
      return this.initialize(data);
    }
  };

  Config.prototype.get = function (keyAlias) {
    if (keyAlias && typeof keyAlias === 'string') {
      var keys = keyAlias.split('.');
      var data = this._data;
      for (var key in keys) {
        if (keys.hasOwnProperty(key) && keys[key]) {
          var index = keys[key];

          if (data.hasOwnProperty(index)) {
            data = data[index];
          } else {
            return;
          }
        }
      }

      return data;
    }
  };

  namespace('app.libs.Config', Config);

})();