(function () {
  "use strict";

  var namespace = require('ns3');
  var fs = require('fs');
  var path = require('path');

  var sendFilesHelper = {
    /**
     * Отправка файла по http
     * @param {object} res          Node express Respose
     * @param {string} filename     Filename - file or dir (full path)
     * @param {string} rootDir      Root dir for protection (full path)
     * @param {string} rootHome     Root home url, ex: "/get/testp/1234567/"
     * @returns {undefined}
     */
    send: function (res, filename, rootDir, rootHome) {
      
      var rootDir = path.join(rootDir);
      var filename = path.join(filename);
      if (rootDir && filename && filename.indexOf(rootDir)===0) {
        this._sendFile(res, filename, rootHome);
      } else {
        res.sendStatus(403);
      }
    },
    // Отправка файла клиенту HTTP
    _sendFile: function (res, filename, rootHome) {
      var $this = this;
      fs.lstat(filename, function (err, stats) {
        if (err) {
          // not found
          res.sendStatus(404);
          return;
        }

        if (stats.isFile()) {
          var options = {
            root: path.dirname(filename),
            dotfiles: 'allow',
            headers: {
              'x-timestamp': Date.now(),
              'x-sent': true
            }
          };
          res.sendFile(path.basename(filename), options, function (err) {
            if (err) {
              res.status(err.status).end();
            }
            else {
              res.end();
            }
          });
          return;
        }
        if (stats.isDirectory()) {
          fs.readdir(filename, function (err, files) {
            if (err) {
              res.sendStatus(403);
            } else {
              $this._sendDirectory(res, filename, files, rootHome);
            }
          });
          return;
        }
        // неизвестная ошибка
        res.sendStatus(500);
      })
    },
    _sendDirectory: function (res, dirPath, files, rootHome, html) {
      var $this = this;
      if (!html) {
        html = '<link rel="stylesheet" href="//cdnjs.cloudflare.com/ajax/libs/font-awesome/4.4.0/css/font-awesome.min.css">';
        html += "\n<style>.fsize { color:silver; } ul {list-style-type:none;} ul li {margin-bottom:2px;}</style>";
        html += "<ul>";
      }
      var file = files.shift();
      if (file) {
        fs.lstat(dirPath + '/' + file, function (err, stats) {
          if (!err) {
            var is_dir = stats.isDirectory();
            var size = false;
            if (!is_dir) {
              var size = stats.size;
              if (size > 512) {
                if (size) {
                  size = Math.round(size / 10.24) / 100;
                  if (size > 512) {
                    size = Math.round(size / 10.24) / 100;
                    size += 'Mb';
                  } else {
                    size += 'Kb';
                  }
                }
              } else {
                size += 'b';
              }
            }
            size = size === false ? '' : ' <span class="fsize"> - ' + size + '</span> ';
            html += '<li><i class="fa ' + (is_dir ? 'fa-folder' : 'fa-file-o') + '"></i>'
                +'<a href="' + rootHome + file + (is_dir ? '/' : '') + '">' + file + '</a>' + size + '</li>';
          }
          $this._sendDirectory(res, dirPath, files, rootHome, html);
        });
      } else {
        html += '</ul>';
        res.send(html);
      }
    }
  };

  namespace('app.libs.sendFilesHelper', sendFilesHelper);

})();